// This class implements a counter of arbitrary base and digits.
// 
// Codon Optimizer - a reference implementation of the CAI optimization 
// and motif engineering algorithm by Anne Condon and Chris Thachuk. 
//
// Copyright (C) 2012 Chris Thachuk
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _COUNTER_H_
#define _COUNTER_H_

namespace oligodesign {

class Counter {
public:
  // B is the base of the counter, D is the number of digits
  Counter(int B, int D) {
    _B = B;
    _D = D;
    _cntr = new unsigned int[_D + 1];
    Reset();
    _int_max = (int)pow((double)_B, (double)_D) - 1;
  }

  Counter( const Counter &cntr ) {
    _B = cntr._B;
    _D = cntr._D;
    _cntr = new unsigned int[_D + 1];
    _int_max = (int)pow((double)_B, (double)_D) - 1;
    for(unsigned int i=0; i<=_D; i++) _cntr[i] = cntr._cntr[i];
    _int_value = cntr._int_value;
  }

  Counter& operator=(const Counter &rhs) {
    // Check for self-assignment!
    if (this == &rhs)      
      return *this;        
    
    delete [] _cntr;
    
    _B = rhs._B;
    _D = rhs._D;
    _cntr = new unsigned int[_D + 1];
    _int_max = rhs._int_max;
    for(unsigned int i=0; i<=_D; i++) _cntr[i] = rhs._cntr[i];
    _int_value = rhs._int_value;
    
    return *this;
  }
  
  ~Counter()
  {
    delete [] _cntr;
  }
  
  void Reset() 
  {
    for(unsigned int i=0; i<=_D; i++) _cntr[i] = 0;
    _int_value = 0;
  }

  void Increment() {
    bool did_increment = false;
  
    for(unsigned int i=0; i<=_D; i++) {
      if( _cntr[i] < _B - 1 ) {
        _cntr[i]++;
        did_increment = true;
        break;
      } else {
        _cntr[i] = 0;
      }
    }
    _int_value++;
    if (! did_increment ) _int_value = 0;
  }

  void LeftShift() {
    _int_value = 0;
    for(unsigned int i=_D-1; i>0; i--) {
      _cntr[i] = _cntr[i-1];
      _int_value += _cntr[i] * (int)pow((double)_B, (double)i);
    }
    _cntr[0] = 0;
  }

  void SetDigit(int digit, int val) {
    if ((digit < 0) || (digit >= _D) || (val < 0) || (val >= _B)) {
      return;
    }

    int old_val = _cntr[digit];
    if (old_val == val) {
      return;
    }
    
    _cntr[digit] = val;
    int val_diff = val - old_val;
    int diff = (int)pow((double)(val_diff), (double)(digit+1));
    _int_value += ((old_val > val) ? -diff : diff);
  }

  unsigned int Digit(int index) {
    if( (0 <= index) && (index < _D) ) return _cntr[index];
    return 0;
  }

  unsigned int IntValue() {
    return _int_value;
  }

  unsigned int IntMax() {
    return _int_max;
  }

private:
  unsigned int *_cntr;
  unsigned int _int_value;
  unsigned int _int_max;
  unsigned int _B;
  unsigned int _D;
};

} // end namespace oligodesign

#endif /* _COUNTER_H_ */
